:root {
    --mainNavColor: #251395;
    --linkColor: #00a1a1;
    --sd-color-primary: var(--mainNavColor);
    --sd-color-primary-highlight: #4427f6;
    --sd-color-secondary: var(--linkColor);
}

div#top_nav nav h1 a {
    color: #ffd01d;
}
